/**
 * @author Romain Ferrari, romain.ferrari[AT]thalesgroup.com
 *
 */
package com.sun.xacml.finder.impl;

import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.rmi.RemoteException;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import com.sun.xacml.EvaluationCtx;
import com.sun.xacml.attr.AttributeDesignator;
import com.sun.xacml.attr.AttributeValue;
import com.sun.xacml.attr.BagAttribute;
import com.sun.xacml.attr.StringAttribute;
import com.sun.xacml.cond.EvaluationResult;
import com.sun.xacml.finder.AttributeFinderModule;
import com.thalesgroup.fortress.MonServicePortBindingStub;
import com.thalesgroup.fortress.Session;

public class AttributeRoleFinder extends AttributeFinderModule {

	private static final org.apache.log4j.Logger LOG4JLOGGER = org.apache.log4j.Logger.getLogger(AttributeRoleFinder.class);
	
	private String url;
	private String fortressAttribute;
	private String substituteValue;
	private String attributeSupportedId;
	
	private URL endpoint = null ;	
	private StringAttribute subjectVal;
	
	public AttributeRoleFinder(){
		LOG4JLOGGER.debug("Initialisation of the Attribute Role Connector for AuthZForce without args");
	}
	
	public AttributeRoleFinder(List args){		
		LOG4JLOGGER.debug("Initialisation of the Attribute Role Connector for AuthZForce");
		Iterator it = args.iterator();
		while (it.hasNext()){
			Object val = it.next();
			if (val instanceof HashMap){
				HashMap map= (HashMap) val;
				if (map.containsKey("url")){
					url = (String) map.get("url");	
				}
				else if (map.containsKey("attributeSupportedId")){
					attributeSupportedId = (String) map.get("attributeSupportedId");	
				}
				else if (map.containsKey("fortressAttribute")){
					fortressAttribute = (String) map.get("fortressAttribute");	
				}
				else if (map.containsKey("substituteValue")){
					substituteValue = (String) map.get("substituteValue");	
				}
			}	
		}
		LOG4JLOGGER.debug("URL = "+url);
		LOG4JLOGGER.debug("attributeSupportedId = "+attributeSupportedId);
		LOG4JLOGGER.debug("fortressAttribute = "+fortressAttribute);
		LOG4JLOGGER.debug("substituteValue = "+substituteValue);
		if (url != null)
		{
			try {
				endpoint = new URL(url);
			} catch (MalformedURLException e) {
				// TODO Auto-generated catch block
				LOG4JLOGGER.error(e.getLocalizedMessage());
			}
		}
		else {
			LOG4JLOGGER.error("Error in endpoint construction, please set the \"url\" parameter in config.xml");
		}
	}
	
    // always return true, since this is a feature we always support
    public boolean isDesignatorSupported() {
        return true;
    }

    // return a single identifier that shows support for environment attrs
    public Set<Integer> getSupportedDesignatorTypes() {
        Set<Integer> set = new HashSet<Integer>();
        
        set.add(AttributeDesignator.SUBJECT_TARGET);
        return set;
    }
    
    /**
     * 
     * This method grab the user's role(s) by asking Fortress WebService
     * We just need an userId to work with
     * 
     */
	public EvaluationResult findAttribute(URI attributeType,
                                          URI attributeId,
                                          URI issuer, URI subjectCategory,
                                          EvaluationCtx context,
                                          int designatorType) {
    	LOG4JLOGGER.debug("Call of AttributeRoleFinder to resolv subject-role attribute");
    	URI subjectCategoryUri = null;	
    	URI subjectId = null;
    	URI subjectType = null;
    	
    	try {
			subjectCategoryUri = new URI("urn:oasis:names:tc:xacml:1.0:subject-category:access-subject");
			subjectId = new URI("urn:oasis:names:tc:xacml:1.0:subject:subject-id");
			subjectType = new URI("http://www.w3.org/2001/XMLSchema#string");
		} catch (URISyntaxException e1) {
			LOG4JLOGGER.fatal("An exception occured : " + e1.getLocalizedMessage());
		}
    	
		AttributeValue subject = context.getSubjectAttribute(subjectType, subjectId, subjectCategoryUri).getAttributeValue();
		if (subject.isBag()) {
			BagAttribute attrBag = (BagAttribute) subject;
			Iterator it = attrBag.getValue().iterator();
			while (it.hasNext()) {
				subjectVal = (StringAttribute) it.next();
			}
			LOG4JLOGGER.debug("Search attribute : " + attributeId + " for user : " + subjectVal.getValue());
		}
		// Test subject value
		if (subjectVal.getValue() == null) {
			LOG4JLOGGER.error("Subject value is Null, Unable to retrieve fortress-role");
			return new EvaluationResult(BagAttribute.createEmptyBag(attributeType));
		}
    	
		Set<StringAttribute> set = new HashSet<StringAttribute>();
		
		javax.xml.rpc.Service service = null;		
		
		MonServicePortBindingStub client = null;
		Session ses = new Session();

		
		try {
			client = new MonServicePortBindingStub(endpoint, service);			
				
			/**
			 * We do not need a password here (password control disabled in Fortress' WebService)
			 * TODO: create a new method with the ability to do a password check. Fortress core need to be reviewed to achieve that
			 */
			ses = client.authenticateUser(subjectVal.getValue(), "");			
			// Test subject value
			if (ses.getUserId() == null) {
				LOG4JLOGGER.error("No roles availables for this User");
				return new EvaluationResult(
						BagAttribute.createEmptyBag(attributeType));
			}
		} catch (RemoteException e) {
			// TODO: Write comments and an explanation why authenticateUser has failed
			LOG4JLOGGER.error(e.getLocalizedMessage());
			return new EvaluationResult(BagAttribute.createEmptyBag(attributeType));
		}
		
        // make sure this is an Subject attribute
        if (designatorType != AttributeDesignator.SUBJECT_TARGET)
        {
            return new EvaluationResult(BagAttribute.createEmptyBag(attributeType));
        }
        
        // make sure they're asking for our identifier
        if (! attributeId.toString().equals("fortress-role"))
        {
            return new EvaluationResult(BagAttribute.createEmptyBag(attributeType));
        }
                                        
        // make sure they're asking for an String return value
        if (! attributeType.toString().equals(StringAttribute.identifier))
        {
            return new EvaluationResult(BagAttribute.createEmptyBag(attributeType));
        }

        // Fetch and return that/those value(s)
        for (int i = 0; i < ses.getRoles().length; i++)
        {
        	set.add(new StringAttribute(ses.getRoles(i).getName()));
        }
        
        return new EvaluationResult(new BagAttribute(attributeType, set));
    }
}
